/*
 * in.swhoisd.h
 *
 * Copyright  2001 Dan Anderson <dan@drydog.com> All rights reserved.
 * See file COPYING for licensing terms.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY, to the extent permitted by law; without even the
 * implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 *
 * $Id: in.swhoisd.h,v 1.3 2001/07/04 06:13:40 dan Exp dan $
 *
 */

#ifndef	_SYSUTILS_H
#define	_SYSUTILS_H

#include "config.h"

#define	SWHOISD_BANNER		PACKAGE " " VERSION


/*
 * nogroup (or nobody, if there's no "nogroup") value if getgrnam_r() fails:
 * On some systems, setgid() fails if GID > 60000.
 * Some values obtained from http://unix-guide.addr.com/unixguide.shtml
 */
#ifndef MYUID
#define	MYUID	65534
#endif
#ifndef MYGID
#define	MYGID	65534
#endif

#define	MAXLINE			1024	/* for buffering purposes */
#define	MYNAME			"in.swhoisd"
#define	NLS_DOMAIN		"swhoisd" /* for GNU gettext */

#define	SYSCONFDIR 		"/etc"
#define	SYSLOGDIR 		"/var/log"
#define	DAEMON_LOG_FILENAME	SYSLOGDIR "/swhoisd.log"

/* We use the HTTPD "Common Logfile Format"'s time stamp format: */
#define	TIMESTAMP_FORMAT	"%d/%b/%Y:%H:%M:%M"

#include <sys/types.h>
#include <stdio.h>		/* FILE */

/* Socket stuff */
#include <sys/socket.h>		/* getpeername() */
#include <netinet/in.h>		/* struct sockaddr_in */
#include <netdb.h>		/* gethostbyaddr() */

/*
 * Internationalization (I18N)
 */
#include <libintl.h>	/* I18N */
#include <locale.h>	/* I18N */

#ifndef LOCALEDIR
#if defined __sun || defined __hpux
#define	LOCALEDIR "/usr/lib/locale"
#elif defined __linux || defined __freebsd
#define	LOCALEDIR "/usr/share/locale"
#elif defined __aix
#define	LOCALEDIR "/usr/lib/nls/loc"
#else /* unknown--use standard GNU gettext location */
#error "You need to define LOCALEDIR for this system"
#define	LOCALEDIR "/usr/local/share/locale"
#endif
#endif

#define	_(string)	gettext((string))	/* I18N */
#define	N_(string)	(string)		/* No I18N */

/*
 * Selected HTTP error codes (for emulating HTTPD CLF log file)
 */
#define	HTTP_OK				200
#define	HTTP_NO_CONTENT			204
#define	HTTP_MOVED_PERMANENTLY		301
#define	HTTP_MOVED_TEMPORARILY		302
#define	HTTP_BAD_REQUEST		400
#define	HTTP_UNAUTHORIZED		401
#define	HTTP_FORBIDDEN			403
#define	HTTP_NOT_FOUND			404
#define	HTTP_REQUEST_TIME_OUT		408
#define	HTTP_BAD_REQUEST		400
#define	HTTP_REQUEST_ENTITY_TOO_LARGE	413
#define	HTTP_RANGE_NOT_SATISFIABLE	416
#define	HTTP_INTERNAL_SERVER_ERROR	500
#define	HTTP_NOT_IMPLEMENTED		501
#define	HTTP_INSUFFICIENT_STORAGE	507

#ifndef HAVE_STRLCPY
#ifdef HAVE_G_STRLCPY
/* glib g_strlcpy(), if available, emulates strlcpy(): */
#define	strlcpy(dest, src, dest_size) g_strlcpy((dest), (src), (dest_size))
#endif
size_t strlcpy(char *dest, const char *src, size_t dest_size);
#endif

/*
 * in.swhoisd.c
 */

/* Saved command line options (or default values): */
extern int	delay_time;		/* -d, in seconds */
extern int	enable_logging;		/* -l flag (-n disables) */
extern int	standalone_mode;	/* -s flag (-i disables) */
extern uid_t	desired_uid;		/* -u flag */
extern gid_t	desired_gid;		/* -u flag */

/* Perform the requested whois operation: */
int do_whois(FILE *in_fd, FILE *out_fd, char *ident);


/*
 * sysutils.c
 */

/* Return remote user name, given socket structures. */
int rfc1413(struct sockaddr_in *rmt_sin, struct sockaddr_in *our_sin,
		char *dest, unsigned dest_len);


/*
 * sysutils.c
 */

/* Daemonize ourself by detaching from the controlling terminal: */
int daemonize(void);

/* Initialization performed after daemonizing: */
int initialize1(void);

/* Initialization performed after sockets are open: */
int initialize2(void);

/* Return 1 if a socket, 0 if not a socket, -1 on error: */
int issocket(int sockfd);

/* Get the hostname (or IP address) of the connecting client: */
char *get_client_hostname(int sockfd);

/* Run in daemonized standalone mode: */
int run_standalone(void);

/* Get the current time and format and return the result in a string: */
char *get_timestamp(char *buffer, int buffer_size);

/* Print fprint-like to syslog: */
void print_syslog(int priority, char *fmt, ...);

/* Print fprint-like to the Daemon logfile: */
void print_daemonlog(char *fmt, ...);

/* Perform cleanup functions, such as closing logs, sockets, and files: */
void cleanup(void);

/* Get the "nobody" (or if none, "nouser") UID value for this system: */
uid_t get_nobody_uid(void);

/* Get the "nogroup" (or if none, "nobody") GID value for this system: */
gid_t get_nogroup_gid(void);

/* Add an entry to the daemon log: */
void log_to_daemonlog(int sock, char *request, char *ident, int status,
			int len);
#endif
