/*
 * main.c
 *
 * Copyright  2001 Dan Anderson <dan@drydog.com> All rights reserved.
 * See file COPYING for licensing terms.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY, to the extent permitted by law; without even the
 * implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 */


#ifndef lint
static char rcsid[] = "@(#) $Id: main.c,v 1.2 2001/07/04 06:14:22 dan Exp $";
#endif

#define	DEFAULT_DELAY_TIME	2 /* seconds */


#include "config.h"
#include "in.swhoisd.h"
#include <unistd.h>		/* getopt() */
#include <syslog.h>		/* LOG_ERR, syslog() */


#ifndef lint
static char swhoisdid[] = "@(#) " SWHOISD_BANNER;
#endif


/* Command line options: */
int	delay_time = DEFAULT_DELAY_TIME;	/* -d, in seconds */
int	enable_logging = 1;			/* -l flag (-n disables) */
#ifndef INET_ONLY
int	standalone_mode = 1;			/* -s flag (-i disables) */
#else
int	standalone_mode = 0;			/* -s flag (-i disables) */
#endif
uid_t	desired_uid;				/* -u flag */
gid_t	desired_gid;				/* -u flag */


/*
 * Print usage to stderr and exit with a non-zero value.
 */
static void
usage(void) {
	(void) fprintf(stderr, _("Simple Whois Daemon "));
	(void) fputs(SWHOISD_BANNER "\n", stderr);

	(void) fprintf(stderr, _(
"Usage:\n"
"	in.swhoisd [-d delay] [-g gid] [-h] [-i] [-l] [-n] [-s] [-u uid]\n"
"Where:\n"
"      -d    Specify delay in seconds between requests\n"
"            (default is 3 seconds)\n"
"      -g    Set the group ID (GID) used by this daemon\n"
"            (default is nogroup or nobody)\n"
"      -h    Print this usage message\n"
"      -i    Run in inet mode\n"
"            (this is the default if is stdin is a socket)\n"
"      -l    Enable logging (default)\n"
"      -n    Disable logging\n"
"      -s    Run in standalone mode\n"
"            (this is the default if is stdin is *not* a socket)\n"
"      -u    Set the user ID (UID) used by this daemon\n"
"            (default is nobody or nouser)\n"));

	exit(1);
}



/*
 * Entrypoint for program.
 * Called either in standalone mode or inetd mode.
 *
 * inetd mode
 * Since this program is invoked by [x]inetd, there are no command line
 * parameters.
 * There is no stderr, just stdout, which is sent to the whois client.
 *
 * Standalone mode:
 * Not implemented yet.
 */
int
main(int argc, char *argv[]) {
	int c;

	/* Internationalization: */
	(void) setlocale(LC_ALL, ""); /* Specify scope of L10N (everything) */
	(void) bindtextdomain(NLS_DOMAIN, LOCALEDIR); /* Specify L10N path */
	(void) textdomain(NLS_DOMAIN); /* Specify "domain" to use for _() */

	/* Set default values for various options */
	desired_uid = get_nobody_uid();
	desired_gid = get_nogroup_gid();

#ifndef INET_ONLY
	/* If stdin is a socket, default to inetd mode: */
	if (issocket(0)) {
		standalone_mode = 0;
	}
#endif

	/* Parse command line options, if any */
	while ((c = getopt(argc, argv, "d:g:hlnsu:")) != EOF) {
		switch (c) {
		case 'd':
			delay_time = atoi(optarg);
			break;
		case 'g':
			desired_gid = atoi(optarg);
			break;
		case 'l':
			enable_logging = 1;
			break;
		case 'i': /* inetd mode */
			standalone_mode = 0;
			break;
		case 'n': /* disable logging */
			enable_logging = 0;
			break;
		case 's':
#ifndef INET_ONLY
			standalone_mode = 1;
#else
			(void) fprintf(stderr, _("standalone mode disabled"));
			usage();
#endif
			break;
		case 'u':
			desired_uid = atoi(optarg);
			break;
		case 'h': /*FALLTHROUGH*/
		case '?': /*FALLTHROUGH*/
		default:
			usage();
			break;
		}
	}

	/* Run standalone or inetd mode: */
	if (standalone_mode) {
#ifndef INET_ONLY
		run_standalone();

#endif
	} else { /* inetd mode */
		initialize1();
		initialize2();
		(void) do_whois(stdin, stdout, "-");
	}

	/* cleanup */
	cleanup();
	return (0);
}	/* main */
