/*
 * snprintf.c
 *
 * Copyright  2001 Dan Anderson <dan@drydog.com> All rights reserved.
 * See file COPYING for licensing terms.
 *
 * Emulate snprintf() and vsnprintf().
 * Based on source code by the late W. Richard Stevens in his book
 * _UNIX Network Programming_ 2d ed. v.1.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY, to the extent permitted by law; without even the
 * implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 */

/*
 * Throughout the book I use snprintf() because it's safer than sprintf().
 * But as of the time of this writing, not all systems provide this
 * function.  The function below should only be built on those systems
 * that do not provide a real snprintf().
 * The function below just acts like sprintf(); it is not safe, but it
 * tries to detect overflow.
 */

/*
 *	LIMITS OF LIABILITY AND DISCLAIMER OF WARRANTY
 *
 * The author and publisher of the book "UNIX Network Programming" have
 * used their best efforts in preparing this software.  These efforts
 * include the development, research, and testing of the theories and
 * programs to determine their effectiveness.  The author and publisher
 * make no warranty of any kind, express or implied, with regard to
 * these programs or the documentation contained in the book. The author
 * and publisher shall not be liable in any event for incidental or
 * consequential damages in connection with, or arising out of, the
 * furnishing, performance, or use of these programs.
 */

#include "config.h"
#include "in.swhoisd.h"
#include <stdarg.h>		/* ANSI C header file */
#include <stdlib.h>		/* exit() */
#include <string.h>		/* strlen() */
#include <syslog.h>		/* LOG_ERR */


int
vsnprintf(char *buf, size_t size, const char *fmt, va_list ap) {
	int		n;

	/* Sigh, some vsprintf's return ptr, not length */
	(void) vsprintf(buf, fmt, ap);

	n = strlen(buf);
	if (n >= size) {
		print_syslog(LOG_ERR, "vsnprintf: '%s' overflowed array"
			" (used %d bytes, buffer length %d bytes); exiting",
			fmt, n, size);
		/* Must exit as the stack is probably hammered: */
		exit(1);
	}

	return (n);
}


int
snprintf(char *buf, size_t size, const char *fmt, ...) {
	int		n;
	va_list		ap;

	va_start(ap, fmt);
	n = vsnprintf(buf, size, fmt, ap);
	va_end(ap);

	return (n);
}
